@@ -3,7 +3,7 @@ BEGIN {
   $App::DuckPAN::Cmd::Server::AUTHORITY = 'cpan:DDG';
 }
 {
-  $App::DuckPAN::Cmd::Server::VERSION = '0.103';
+  $App::DuckPAN::Cmd::Server::VERSION = '0.104';
 }
 # ABSTRACT: Starting up the webserver to test plugins
 
@@ -27,49 +27,58 @@ sub run {
 
 	dir($self->app->cfg->cache_path)->mkpath unless -d $self->app->cfg->cache_path;
 
-	copy(file(dist_dir('App-DuckPAN'),'page_root.html'),file($self->app->cfg->cache_path,'page_root.html')) unless -f file($self->app->cfg->cache_path,'page_root.html');
-	copy(file(dist_dir('App-DuckPAN'),'page_spice.html'),file($self->app->cfg->cache_path,'page_spice.html')) unless -f file($self->app->cfg->cache_path,'page_share.html');
-	copy(file(dist_dir('App-DuckPAN'),'page.css'),file($self->app->cfg->cache_path,'page.css')) unless -f file($self->app->cfg->cache_path,'page.css');
-	copy(file(dist_dir('App-DuckPAN'),'page.js'),file($self->app->cfg->cache_path,'page.js')) unless -f file($self->app->cfg->cache_path,'page.js');
+	my %spice_files = (
+		'page_root.html'         => { name => 'DuckDuckGo HTML', file_path => '/' },
+		'page_spice.html'        => { name => 'DuckDuckGo Spice-Template', file_path => '/?q=duckduckhack-template-for-spice2' },
+		'page.css'               => { name => 'DuckDuckGo CSS', file_path => '/style.css' },
+		'duckduck.js'            => { name => 'DuckDuckGo Javascript', file_path => '/duckduck.js' },
+		'jquery.js'              => { name => 'jQuery', file_path => '/js/jquery/jquery-1.8.2.min.js' },
+		'handlebars.js'          => { name => 'Handlebars.js', file_path => '/js/handlebars-1.0.0-rc.3.js' },
+		'spice2_latest.js'       => { name => 'Spice2.js', file_path => '/spice2/spice2_latest.js' },
+		'spice2_duckpan.js'      => { name => 'Spice2 DuckPAN javascript', file_path => '/spice2/spice2_duckpan.js' }
+	);
 
 	my @blocks = @{$self->app->ddg->get_blocks_from_current_dir(@args)};
 
-	print "\n\nTrying to fetch current versions of the HTML from http://duckduckgo.com/\n\n";
-
 	my $hostname = $self->app->server_hostname;
+	print "\n\nTrying to fetch current versions of the HTML from http://$hostname/\n\n";
 
-	my $fetch_page_root;
-	if ($fetch_page_root = get('http://'.$hostname.'/')) {
-		io(file($self->app->cfg->cache_path,'page_root.html'))->print($self->change_html($fetch_page_root));
-	} else {
-		print "\nRoot fetching failed, will just use cached version..."
-	}
+	foreach my $file_name (keys %spice_files){
+		copy(file(dist_dir('App-DuckPAN'),$file_name),file($self->app->cfg->cache_path,$file_name)) unless -f file($self->app->cfg->cache_path,$file_name);
 
-	my $fetch_page_spice;
-	if ($fetch_page_spice = get('http://'.$hostname.'/?q=duckduckhack-template-for-spice2')) {
-		io(file($self->app->cfg->cache_path,'page_spice.html'))->print($self->change_html($fetch_page_spice));
-	} else {
-		print "\nSpice-Template fetching failed, will just use cached version..."
-	}
+		my $path = $spice_files{$file_name}{'file_path'};
+		my $url = 'http://'.$hostname.''.$path;
+		my $res = $self->app->http->request(HTTP::Request->new(GET => $url));
 
-	my $fetch_page_css;
-	if ($fetch_page_css = get('http://'.$hostname.'/style.css')) {
-		io(file($self->app->cfg->cache_path,'page.css'))->print($self->change_css($fetch_page_css));
-	} else {
-		print "\nCSS fetching failed, will just use cached version..."
-	}
+		if ($res->is_success){
 
-	my $fetch_page_js;
-	if ($fetch_page_js = get('http://'.$hostname.'/duckduck.js')) {
-		io(file($self->app->cfg->cache_path,'page.js'))->print($self->change_js($fetch_page_js));
-	} else {
-		print "\nJavaScript fetching failed, will just use cached version..."
+				my $content = $res->decoded_content(charset => 'none');
+
+				if ($file_name =~ m/\.js$/){
+					io(file($self->app->cfg->cache_path,$file_name))->print($self->change_js($content));
+				} elsif  ($file_name =~ m/\.css$/){
+					io(file($self->app->cfg->cache_path,$file_name))->print($self->change_css($content));
+				} else {
+					io(file($self->app->cfg->cache_path,$file_name))->print($self->change_html($content));
+				}
+		} else {
+			#print $res->status_line, "\n";
+			print "\n".$spice_files{$file_name}{'name'}." fetching failed, will just use cached version...";
+		}
 	}
 
 	my $page_root = io(file($self->app->cfg->cache_path,'page_root.html'))->slurp;
+
 	my $page_spice = io(file($self->app->cfg->cache_path,'page_spice.html'))->slurp;
 	my $page_css = io(file($self->app->cfg->cache_path,'page.css'))->slurp;
-	my $page_js = io(file($self->app->cfg->cache_path,'page.js'))->slurp;
+
+	# Concatenate all JS files
+	# Order matters because of dependencies
+	my $page_js = io(file($self->app->cfg->cache_path,'duckduck.js'))->slurp;
+	$page_js .= io(file($self->app->cfg->cache_path,'jquery.js'))->slurp;
+	$page_js .= io(file($self->app->cfg->cache_path,'handlebars.js'))->slurp;
+	$page_js .= io(file($self->app->cfg->cache_path,'spice2_latest.js'))->slurp;
+	$page_js .= io(file($self->app->cfg->cache_path,'spice2_duckpan.js'))->slurp;
 
 	print "\n\nStarting up webserver...";
 	print "\n\nYou can stop the webserver with Ctrl-C";
@@ -83,6 +92,7 @@ sub run {
 		page_spice => $page_spice,
 		page_css => $page_css,
 		page_js => $page_js,
+		server_hostname => $hostname,
 	);
 	my $runner = Plack::Runner->new(
 		#loader => 'Restarter',
@@ -162,6 +172,7 @@ sub change_html {
 1;
 
 __END__
+
 =pod
 
 =head1 NAME
@@ -170,7 +181,7 @@ App::DuckPAN::Cmd::Server - Starting up the webserver to test plugins
 
 =head1 VERSION
 
-version 0.103
+version 0.104
 
 =head1 AUTHOR
 
@@ -184,4 +195,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
